#ifndef _Dialog_h
#define _Dialog_h

/*
 *  subtitle editor
 *
 *	http://kitone.free.fr/subtitleeditor/
 *
 *	Copyright @ 2005-2006, kitone
 *
 *  Contact: kitone at free dot fr
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *  \file
 *  \brief 
 *  \author kitone (kitone at free dot fr)
 */

#include <gtkmm/dialog.h>
#include <gtkmm/label.h>
#include <gtkmm/entry.h>
#include <gtkmm/table.h>
#include <gtkmm/filechooserdialog.h>
#include <gtkmm/spinbutton.h>
#include <gtkmm/aboutdialog.h>
#include <gtkmm/radiobutton.h>
#include <gtkmm/togglebutton.h>
#include <gtkmm/fontbutton.h>
#include <gtkmm/colorbutton.h>
#include <gtkmm/treeview.h>
#include <gtkmm/comboboxentrytext.h>
#include <gtkmm/comboboxtext.h>

#include <libglademm/xml.h>

#include "SubtitleTime.h"
#include "SubtitleModel.h"
#include "StyleModel.h"
#include "Document.h"
#include "SubtitleView.h"
#include <iostream>
#include "DialogFileChooser.h"

/*
 *
 */
class GtkSubtitleTime : public Gtk::Table
{
public:
  GtkSubtitleTime();

  void set_value(int h, int m, int s, int ms);
  void set_value(const SubtitleTime &time);

  SubtitleTime get_value();

protected:
  Gtk::Label      m_labelHours;
  Gtk::SpinButton m_spinHours;
  Gtk::Label      m_labelMins;
  Gtk::SpinButton m_spinMins;
  Gtk::Label      m_labelSecs;
  Gtk::SpinButton m_spinSecs;
  Gtk::Label      m_labelMSecs;
  Gtk::SpinButton m_spinMSecs;
};



/*
 *
 */
class DialogFind : public Gtk::Dialog
{
public:
  DialogFind(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
	~DialogFind();

	void init(Document *doc);

	void findNext();

	/*
	 *	iter = pointe sur la ligne trouver
	 *	fstart = position du mots dans le texte s'il est trouver
	 */
	bool findNext(Gtk::TreeIter &iter, Glib::ustring::size_type &fstart);

protected:
  Glib::ustring getFindText();
	bool getMatchCase();
	bool getMatchEntireWordOnly();

protected:
	Gtk::Entry* m_entryText;
	Gtk::CheckButton* m_checkMatchCase;
	Gtk::CheckButton* m_checkMatchEntireWordOnly;
	bool restart;
};



/*
 *
 */
class DialogFindAndReplace : public Gtk::Dialog
{
public:
  DialogFindAndReplace(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);

  Glib::ustring getFindText();
  Glib::ustring getReplaceText();

  enum RESPONSE
  {
    CLOSE=0,
    REPLACE_ALL=1,
    REPLACE=2,
    FIND=3
  };
protected:
  Gtk::Entry*  m_entryFind;

  Gtk::Entry*  m_entryReplace;
};





/*
class DialogOpen : public Gtk::FileChooserDialog
{
public:
  DialogOpen(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
};
*/

/*
 *
 */
class DialogOpenSubtitle : public DialogFileChooser
{
public:
  DialogOpenSubtitle();
	~DialogOpenSubtitle();
};



/*
 *
 */
class DialogSaveSubtitle : public DialogFileChooser
{
public:
  DialogSaveSubtitle();
	~DialogSaveSubtitle();
};



/*
 *
 */
class DialogOpenMovie : public Gtk::FileChooserDialog
{
public:
  DialogOpenMovie();
	~DialogOpenMovie();
};


/*
 *
 */
class DialogOpenMedia : public Gtk::FileChooserDialog
{
public:
  DialogOpenMedia();
	~DialogOpenMedia();
};

/*
 *
 */
class DialogSaveWaveform : public Gtk::FileChooserDialog
{
public:
  DialogSaveWaveform();
	~DialogSaveWaveform();
};



/*
 *
 */
class DialogImportText : public DialogFileChooser
{
public:
  DialogImportText();
};



/*
 *
 */
class DialogExportText : public DialogFileChooser
{
public:
  DialogExportText();
};





/*
 *
 */
class DialogMoveSubtitle : public Gtk::Dialog
{
public:
  DialogMoveSubtitle();
  DialogMoveSubtitle(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);

  void set_value(const SubtitleTime &start);
  SubtitleTime get_value();

protected:
  Gtk::Table	  m_table;
  Gtk::Label	  m_labelDest;
  GtkSubtitleTime m_timer;
  Gtk::Label	  m_labelOption;
  Gtk::RadioButton  m_radioAll;
  Gtk::RadioButton  m_radioCurrent;
};


/*
 *
 */
class DialogSetAllEndTime : public Gtk::Dialog
{
public:
  DialogSetAllEndTime();

  enum Type
  {
    ADD,
    SET,
  };

  Type		get_type();
  SubtitleTime	get_time();
  
protected:
  Gtk::Table	  m_table;

  Gtk::Label	  m_labelRadio;
  Gtk::RadioButton  m_radioAdd;
  Gtk::RadioButton  m_radioSet;

  Gtk::Label	  m_label;
  GtkSubtitleTime m_time;
};


#endif//_Dialog_h
