/*
 *  subtitle editor
 *
 *	http://kitone.free.fr/subtitleeditor/
 *
 *	Copyright @ 2005-2006, kitone
 *
 *  Contact: kitone at free dot fr
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *  \file
 *  \brief 
 *  \author kitone (kitone at free dot fr)
 */

#include "DialogPreferences.h"
#include "Config.h"


DialogPreferences::DialogPreferences(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
:Gtk::Dialog(cobject)
{
	refGlade->get_widget_derived("combo-player", m_comboPlayer);
	refGlade->get_widget("entry-command", m_entryCommand);

	m_comboPlayer->signal_changed().connect(
			sigc::mem_fun(*this, &DialogPreferences::on_combo_player_changed));

	m_comboPlayer->get_entry()->signal_activate().connect(
			sigc::mem_fun(*this, &DialogPreferences::on_combo_entry_activate));

	m_entryCommand->signal_activate().connect(
			sigc::mem_fun(*this, &DialogPreferences::on_entry_command_changed));

	//
	Config &cfg = Config::getInstance();

	std::list<Glib::ustring> list_player;

	if(cfg.get_keys("video-player", list_player))
	{
		std::list<Glib::ustring>::const_iterator it;
		for(it=list_player.begin(); it!=list_player.end(); ++it)
		{
			if(*it != "default")
				m_comboPlayer->append_text(*it);
		}

		Glib::ustring default_player;
		if(cfg.get_value_string("video-player", "default", default_player))
			m_comboPlayer->set_active_text(default_player);
	}
}

void DialogPreferences::on_combo_player_changed()
{
	Glib::ustring text = m_comboPlayer->get_active_text();
	Glib::ustring cmd;


	if(text.empty())
		return;

	Config &cfg = Config::getInstance();
	
	if(cfg.get_value_string("video-player", text, cmd))
	{
		m_entryCommand->set_text(cmd);
	}
	// default
	cfg.set_value_string("video-player", "default", text);
}

void DialogPreferences::on_combo_entry_activate()
{
	Glib::ustring player = m_comboPlayer->get_entry()->get_text();
	Glib::ustring cmd = m_entryCommand->get_text();

	m_comboPlayer->append_text(player);

	Config &cfg = Config::getInstance();
	cfg.set_value_string("video-player", player, cmd);

	m_entryCommand->grab_focus();

	// default
	cfg.set_value_string("video-player", "default", player);
}

void DialogPreferences::on_entry_command_changed()
{
	Glib::ustring player = m_comboPlayer->get_active_text();
	Glib::ustring cmd = m_entryCommand->get_text();

	Config &cfg = Config::getInstance();

	cfg.set_value_string("video-player", player, cmd);
}
