/*
 *  subtitle editor
 *
 *	http://kitone.free.fr/subtitleeditor/
 *
 *	Copyright @ 2005-2006, kitone
 *
 *  Contact: kitone at free dot fr
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See gpl.txt for more information regarding the GNU General Public License.
 *
 *
 *  \file
 *  \brief 
 *  \author kitone (kitone at free dot fr)
 */


#include "GstLaunch.h"
#include <iostream>

//#define TRACE() std::cerr << __PRETTY_FUNCTION__ << std::endl;
#define TRACE()


namespace Gst
{
	/*
	 *	retourne le temps en string par rapport au temps nsecs (gstreamer)
	 */
	Glib::ustring time_to_string (gint64 time)
	{
		gchar *str = g_strdup_printf ("%u:%02u:%02u",
		    (guint) (time / (GST_SECOND * 60 * 60)),
		    (guint) ((time / (GST_SECOND * 60)) % 60),
		    (guint) ((time / GST_SECOND) % 60));

		Glib::ustring res(str);
		g_free(str);
		return res;
	}
}


gboolean __static_bus_message(GstBus *bus, GstMessage *message, gpointer data)
{
	return ((GstLaunch*)data)->bus_message(bus, message);
}

/*

bool pool_for_state_change_full(GstElement *element, GstState state, GError **error, gint64 timeout)
{
	GstBus *bus = NULL;
	GstMessageType events;

	bus = gst_element_get_bus(element);

	events = (GstMessageType)(GST_MESSAGE_STATE_CHANGED | GST_MESSAGE_ERROR | GST_MESSAGE_EOS);

	while(true)
	{
		GstMessage *message = NULL;
		GstElement *src = NULL;

		message = gst_bus_poll(bus, events, timeout);

		if(!message)
			goto timed_out;

		src = (GstElement*)GST_MESSAGE_SRC(message);

		switch(GST_MESSAGE_TYPE(message))
		{
			case GST_MESSAGE_STATE_CHANGED:
				{
					GstState sold, snew, spending;

					if(src == element)
					{
						gst_message_parse_state_changed(message, &sold, &snew, &spending);
						if(snew == state)
						{
							gst_message_unref(message);
							goto success;
						}
					}
				}break;

			case GST_MESSAGE_ERROR:
				{
					gchar *debug = NULL;
					GError *gsterror = NULL;

					gst_message_parse_error(message, &gsterror, &debug);

					if(error)
					{
						// *error = 
					}
					else
					{
						g_warning("Error: %s (%s)", gsterror->message, debug);
					}

					gst_message_unref(message);
					g_error_free(gsterror);
					g_free(debug);
				
					goto error;
				}break;
			
			case GST_MESSAGE_EOS:
				{
					gst_message_unref(message);
					goto error;
				}break;

			default:
				gst_message_unref(message);
				break;
		}
	}

success:
	return true;

timed_out:
	return true;

error:
	return false;

}
*/

static void
event_loop (GstElement * pipe)
{
  GstBus *bus;
  GstMessage *message = NULL;

  bus = gst_element_get_bus (GST_ELEMENT (pipe));

  while (TRUE) {
    message = gst_bus_poll (bus, (GstMessageType)(GST_MESSAGE_STATE_CHANGED | GST_MESSAGE_ERROR | GST_MESSAGE_EOS), GST_SECOND/4);

    //g_assert (message != NULL);
		if(!message)
			return;

    switch (message->type) {
      case GST_MESSAGE_EOS:
        gst_message_unref (message);
        return;
			case GST_MESSAGE_STATE_CHANGED:
				gst_message_unref (message);
        return;
				break;
      case GST_MESSAGE_WARNING:
      case GST_MESSAGE_ERROR:{
        GError *gerror;
        gchar *debug;

        gst_message_parse_error (message, &gerror, &debug);
        gst_object_default_error (GST_MESSAGE_SRC (message), gerror, debug);
        gst_message_unref (message);
        g_error_free (gerror);
        g_free (debug);
        return;
      }
      default:
        gst_message_unref (message);
        break;
    }
  }
}


/*
 *	constructeur
 *	time permet de regler le temps en msecs
 *	pour l'envoi d'information (timeout 
 *	utiliser pendant la lecture
 */
GstLaunch::GstLaunch(guint time) 
:m_pipeline(NULL), m_src(NULL), m_bus(NULL), m_timeout_t(time)
{
	TRACE();
}

/*
 *	destructeur
 *	disconnect signal_timeout et init le pipeline (GST_STATE_NULL)
 */
GstLaunch::~GstLaunch()
{
	TRACE();
	
	if(m_connection)
		m_connection.disconnect();

	if(m_pipeline)
		gst_element_set_state(GST_ELEMENT(m_pipeline), GST_STATE_NULL);
}

/*
 *	s'utilise comme gst-launch
 *	l'element principale doit s'appeler "src"
 *	exemple : playbin name=src ...
 */
bool GstLaunch::parse(const Glib::ustring &pipe) throw (GstLaunch::Error)
{
	TRACE();

	std::cout << "GstLaunch::parse > " << pipe << std::endl;

	GError *error = NULL;
	m_pipeline = gst_parse_launch(pipe.c_str(), &error);

	if(error || !m_pipeline)
	{
		std::cerr << "GstLaunch::parse > " << error->message << std::endl;
		g_error_free(error);

		throw PARSE_ERROR;
	}
	
	m_src = gst_bin_get_by_name(GST_BIN(m_pipeline), "src");

	m_bus = gst_pipeline_get_bus(GST_PIPELINE(m_pipeline));

	gst_bus_add_watch(GST_BUS(m_bus), __static_bus_message, this);

	return true;
}

/*
 *	gestion des messages gstreamer :
 *	GST_MESSAGE_ERROR, GST_MESSAGE_WARNING, 
 *	GST_MESSAGE_EOS, GST_MESSAGE_STATE_CHANGED
 *
 *	cette fonction est important 
 *	il faut l'appeler si on fait deriver cette fonction
 */
bool GstLaunch::bus_message(GstBus *bus, GstMessage *message) throw (GstLaunch::Error)
{
	TRACE();

	if(message->type == GST_MESSAGE_ERROR)
	{
		std::cerr << "GST_MESSAGE_ERROR" << std::endl;

		GError *error = NULL;
		gchar* debug = NULL;

		gst_message_parse_error(message, &error, &debug);

		GST_DEBUG("Error message: %s [%s]", GST_STR_NULL(error->message), GST_STR_NULL(debug));
		std::cerr << "Error message (GST_MESSAGE_ERROR): " << GST_STR_NULL(error->message) << GST_STR_NULL(debug) << std::endl;

		// signal error emit...
		if(m_pipeline)
			gst_element_set_state(GST_ELEMENT(m_pipeline), GST_STATE_NULL);

		g_error_free(error);
		g_free(debug);

		on_error();
		//throw FAILED;
	}
	else if(message->type == GST_MESSAGE_WARNING)
	{
		//std::cerr << "GST_MESSAGE_WARNING" << std::endl;

		GError *error = NULL;
		gchar* debug = NULL;

		gst_message_parse_warning(message, &error, &debug);

		GST_DEBUG("Warning message: %s [%s]", GST_STR_NULL(error->message), GST_STR_NULL(debug));
		std::cerr << "Error message: " << GST_STR_NULL(error->message) << GST_STR_NULL(debug) << std::endl;

    g_warning ("%s [%s]", GST_STR_NULL (error->message), GST_STR_NULL (debug));

		g_error_free(error);
		g_free(debug);
	}
	else if(message->type == GST_MESSAGE_EOS)
	{
		//std::cerr << "GST_MESSAGE_EOS" << std::endl;

		if(m_connection)
			m_connection.disconnect();
		
		on_eos();
	}
	else if(message->type == GST_MESSAGE_STATE_CHANGED)
	{
		//std::cerr << "GST_MESSAGE_STATE_CHANGED" << std::endl;

		GstState old_state, new_state;

		gst_message_parse_state_changed(message, &old_state, &new_state, NULL);

		if(old_state != new_state)
		{
			if(new_state <= GST_STATE_PAUSED)
			{
				if(m_connection)
					m_connection.disconnect();
			}
			else if(new_state > GST_STATE_PAUSED)
			{
				//std::cout << "> paused" << std::endl;

				if(m_timeout_t > 0)
				{
					if(!m_connection)
					{
						/*
						m_timeout = Glib::TimeoutSource::create(m_timeout_t);
						m_timeout->connect(
								sigc::mem_fun(*this, &GstLaunch::on_timeout));
						m_timeout->attach();
						*/
						m_connection = Glib::signal_timeout().connect(
								sigc::mem_fun(*this, &GstLaunch::on_timeout), m_timeout_t);
					}
				}
			}
		}
	}
	return true;
}


/*
 *	le media joue t'il ?
 */
bool GstLaunch::is_playing()
{
	TRACE();

	if(!m_pipeline)
		return false;

	GstState state;
	gst_element_get_state(GST_ELEMENT(m_pipeline), &state, NULL, GST_CLOCK_TIME_NONE);
	return (state == GST_STATE_PLAYING);

//	return GST_STATE(m_pipeline) == GST_STATE_PLAYING;
}

/*
 *	init pipeline a GST_STATE_NULL
 *	(clear)
 */
bool GstLaunch::null()
{
	TRACE();

	if(gst_element_set_state(GST_ELEMENT(m_pipeline), GST_STATE_NULL) == GST_STATE_CHANGE_FAILURE)
	{
		std::cout << __PRETTY_FUNCTION__ << "GST_STATE_CHANGE_FAILURE" << std::endl;
		return false;
	}
	return true;
}

/*
 *	init le pipeline a GST_STATE_READY
 */
bool GstLaunch::ready()
{
	TRACE();

	if(gst_element_set_state(GST_ELEMENT(m_pipeline), GST_STATE_READY) == GST_STATE_CHANGE_FAILURE)
	{
		std::cout << __PRETTY_FUNCTION__ << "GST_STATE_CHANGE_FAILURE" << std::endl;
		return false;
	}
	return true;
/*
	gst_element_set_state(GST_ELEMENT(m_pipeline), GST_STATE_READY);
	//return pool_for_state_change_full(m_pipeline, GST_STATE_READY, NULL, GST_SECOND/4);
	event_loop(m_pipeline);
	return true;
*/
}

/*
 *	init le pipeline a GST_STATE_PLAYING
 */
bool GstLaunch::play()
{
	TRACE();

	if(gst_element_set_state(GST_ELEMENT(m_pipeline), GST_STATE_PLAYING) == GST_STATE_CHANGE_FAILURE)
	{
		std::cout << __PRETTY_FUNCTION__ << "GST_STATE_CHANGE_FAILURE" << std::endl;
		return false;
	}
	return true;
/*
	gst_element_set_state(GST_ELEMENT(m_pipeline), GST_STATE_PLAYING);
	//return pool_for_state_change_full(m_pipeline, GST_STATE_PLAYING, NULL, GST_SECOND/4);
	event_loop(m_pipeline);
	return true;
*/
}

/*
 *	init le pipeline a GST_STATE_PAUSE
 */
bool GstLaunch::pause()
{
	TRACE();

	if(gst_element_set_state(GST_ELEMENT(m_pipeline), GST_STATE_PAUSED) == GST_STATE_CHANGE_FAILURE)
	{
		std::cout << __PRETTY_FUNCTION__ << "GST_STATE_CHANGE_FAILURE" << std::endl;
		return false;
	}
	return true;
/*
	gst_element_set_state(GST_ELEMENT(m_pipeline), GST_STATE_PAUSED);
//	return pool_for_state_change_full(m_pipeline, GST_STATE_PAUSED, NULL, GST_SECOND/4);
	event_loop(m_pipeline);
	return true;
*/
}

/*
 *	retourne la longueur du media (GST_FORMAT_TIME)
 */
gint64 GstLaunch::get_duration()
{
	TRACE();

	g_return_val_if_fail(GST_IS_ELEMENT(m_pipeline), 0);

	gint64 len = 0;
	
	GstFormat fmt = GST_FORMAT_TIME;
	
	if(gst_element_query_duration(GST_ELEMENT(m_pipeline), &fmt, &len))
		return len;
	return 0;
}

/*
 *	retourne la position (lecture) dans le media
 */
gint64 GstLaunch::get_position()
{
	TRACE();

	g_return_val_if_fail(GST_IS_ELEMENT(m_pipeline), 0);

	gint64 pos = 0;
	
	GstFormat fmt = GST_FORMAT_TIME;
	
	if(gst_element_query_position(GST_ELEMENT(m_pipeline), &fmt, &pos))
		return pos;
	return 0;
}

/*
 *	retourne la position en %
 */
gint64 GstLaunch::get_percent()
{
	TRACE();

	g_return_val_if_fail(GST_IS_ELEMENT(m_pipeline), 0);

	gint64 pos = 0;
	
	GstFormat fmt = GST_FORMAT_PERCENT;
	
	if(gst_element_query_position(GST_ELEMENT(m_pipeline), &fmt, &pos))
		return pos;
	return 0;
}

/*
 *	ce deplacer a pos
 */
bool GstLaunch::seek(gint64 pos)
{
	TRACE();

	CLAMP(pos, 0, get_duration());
/*
	bool res = gst_element_seek(m_pipeline, 1.0,
			GST_FORMAT_TIME, (GstSeekFlags)(GST_SEEK_FLAG_FLUSH), 
			GST_SEEK_TYPE_SET, pos,
			GST_SEEK_TYPE_NONE, GST_CLOCK_TIME_NONE);

	return res;
*/
	bool res = gst_element_seek(m_pipeline, 1.0,
			GST_FORMAT_TIME, (GstSeekFlags)(GST_SEEK_FLAG_FLUSH), 
			GST_SEEK_TYPE_SET, pos,
			GST_SEEK_TYPE_SET, get_duration());
/*
	if(res)
	{
		gst_pipeline_set_new_stream_time(GST_PIPELINE(m_pipeline), 0);
		gst_element_get_state(GST_ELEMENT(m_pipeline), NULL,NULL, 40*GST_MSECOND);
	}
	else
		std::cerr << "GstLaunch::seek > seek failed" << std::endl;
*/
	return res;
}


/*
 *	ce deplacer a start et lire jusqu'a end
 *	ne declanche pas play
 */
bool GstLaunch::seek(gint64 start, gint64 end)
{
	TRACE();

	if(start > end)
	{
		gint64 tmp=start;
		start=end;
		end=tmp;
	}
/*
	bool res = gst_element_seek(m_pipeline, 1.0,
			GST_FORMAT_TIME, 
			GST_SEEK_FLAG_FLUSH, 
			GST_SEEK_TYPE_SET, start,
			GST_SEEK_TYPE_SET, end);

	return res;	
*/
	bool res = gst_element_seek(m_pipeline, 1.0,
			GST_FORMAT_TIME, 
			GST_SEEK_FLAG_FLUSH, 
			GST_SEEK_TYPE_SET, start,
			GST_SEEK_TYPE_SET, end);
/*
	if(res)
	{
		gst_pipeline_set_new_stream_time(GST_PIPELINE(m_pipeline), 0);
		gst_element_get_state(GST_ELEMENT(m_pipeline), NULL,NULL, 40*GST_MSECOND);
	}
	else
		std::cerr << "GstLaunch::seek > seek failed" << std::endl;
*/
	return res;

}

/*
 *	fonction callback pour Glib::timeout()
 */
bool GstLaunch::on_timeout()
{
	TRACE();

	m_signal_timeout();//get_position());
	return is_playing();
}


/*
 *	fonction de rappel
 *	seulement pendant la lecture du media
 *	configure la frequence dans le constructeur "timeout"
 */
sigc::signal<void>& GstLaunch::get_signal_timeout()
{
	TRACE();

	return m_signal_timeout;
}

/*
 *	emit si on arrive a la fin du fichier
 */
void GstLaunch::on_eos()
{
	TRACE();
}

/*
 *	emit en cas d'erreur
 */
void GstLaunch::on_error()
{
	TRACE();
}
